"""
Program artifact tools for Spira by Inflectra
"""
from mcp_server_spira.features.programartifacts.tools import (
    milestones,
    capabilities
)


def register_tools(mcp) -> None:
    """
    Register all program artifact tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """
    milestones.register_tools(mcp)
    capabilities.register_tools(mcp)



# General artifact formatting features
from mcp_server_spira.utils.general import get_execution_status_name


def format_task(task) -> str:
    task_info = f"""
## Task [TK:{task['TaskId']}] - {task['Name']}
{'' if task['Description'] is None else task['Description']}
- **Status:** {task['TaskStatusName']}
- **Type:** {task['TaskTypeName']}
- **Priority:** {task['TaskPriorityName']}
- **Due Date:** {task['EndDate']}
"""
    return task_info

def format_incident(incident) -> str:
    incident_info = f"""
## Incident [IN:{incident['IncidentId']}] - {incident['Name']}
{'' if incident['Description'] is None else incident['Description']}
- **Status:** {incident['IncidentStatusName']}
- **Type:** {incident['IncidentTypeName']}
- **Priority:** {incident['PriorityName']}
- **Severity:** {incident['SeverityName']}
- **Due Date:** {incident['StartDate']}
"""
    return incident_info

def format_requirement(requirement) -> str:
    requirement_info = f"""
## Requirement [RQ:{requirement['RequirementId']}] - {requirement['Name']}
{'' if requirement['Description'] is None else requirement['Description']}
- **Status:** {requirement['StatusName']}
- **Type:** {requirement['RequirementTypeName']}
- **Importance:** {requirement['ImportanceName']}
- **Release:** {requirement['ReleaseVersionNumber']}
"""
    return requirement_info

def format_test_case(test_case) -> str:
    test_case_info = f"""
## Test Case [TC:{test_case['TestCaseId']}] - {test_case['Name']}
{'' if test_case['Description'] is None else test_case['Description']}
- **Status:** {test_case['TestCaseStatusName']}
- **Type:** {test_case['TestCaseTypeName']}
- **Priority:** {test_case['TestCasePriorityName']}
- **Last Execution Status:** {test_case['ExecutionStatusName']}
- **Last Executed:** {test_case['ExecutionDate']}
"""
    return test_case_info

def format_test_case_folder(test_case_folder) -> str:
    test_case_folder_info = f"""
# Test Folder: {test_case_folder['Name']}
{'' if test_case_folder['Description'] is None else test_case_folder['Description']}
"""
    return test_case_folder_info

def format_test_set(test_set) -> str:
    test_set_info = f"""
## Test Set [TX:{test_set['TestSetId']}] - {test_set['Name']}
{'' if test_set['Description'] is None else test_set['Description']}
- **Status:** {test_set['TestSetStatusName']}
- **Release:** {test_set['ReleaseVersionNumber']}
- **Recurrence:** {test_set['RecurrenceName']}
- **Due Date:** {test_set['PlannedDate']}
"""
    return test_set_info

def format_test_set_folder(test_set_folder) -> str:
    test_set_folder_info = f"""
# Test Set Folder: {test_set_folder['Name']}
{'' if test_set_folder['Description'] is None else test_set_folder['Description']}
"""
    return test_set_folder_info

def format_product(product) -> str:
    product_info = f"""
## Product [PR:{product['ProjectId']}] - {product['Name']}
{'' if product['Description'] is None else product['Description']}
- **Website:** {product['Website']}
- **Template ID:** [PT:{product['ProjectTemplateId']}]
- **Program ID:** [PG:{product['ProjectGroupId']}]
- **% Complete:** {product['PercentComplete']}%
- **Start Date:** {product['StartDate']}
- **End Date:** {product['EndDate']}
"""
    return product_info

def format_product_template(template) -> str:
    template_info = f"""
## Product Template [PT:{template['ProjectTemplateId']}] - {template['Name']}
{'' if template['Description'] is None else template['Description']}
"""
    return template_info

def format_program(program) -> str:
    program_info = f"""
## Program [PG:{program['ProgramId']}] - {program['Name']}
{'' if program['Description'] is None else program['Description']}
- **Website:** {program['Website']}
- **Product Template ID:** [PT:{program['ProjectTemplateId']}]
- **Portfolio ID:** [PF:{program['PortfolioId']}]
"""
    return program_info

def format_milestone(milestone) -> str:
    milestone_info = f"""
## Milestone [GM:{milestone['MilestoneId']}] - {milestone['Name']}
{'' if milestone['Description'] is None else milestone['Description']}
- **Status:** {milestone['StatusName']}
- **Type:** [PT:{milestone['TypeName']}]
- **% Complete:** {milestone['PercentComplete']}%
- **Start Date:** {milestone['StartDate']}
- **End Date:** {milestone['EndDate']}
"""
    return milestone_info

def format_release(release) -> str:
    release_info = f"""
## Release [RL:{release['ReleaseId']}] - {release['Name']}
{'' if release['Description'] is None else release['Description']}
- **Version #:** {release['VersionNumber']}
- **Status:** {release['ReleaseStatusName']}
- **Type:** [PT:{release['ReleaseTypeName']}]
- **% Complete:** {release['PercentComplete']}%
- **Start Date:** {release['StartDate']}
- **End Date:** {release['EndDate']}
"""
    return release_info

def format_risk(risk) -> str:
    risk_info = f"""
## Risk [RK:{risk['RiskId']}] - {risk['Name']}
{'' if risk['Description'] is None else risk['Description']}
- **Status:** {risk['RiskStatusName']}
- **Type:** {risk['RiskTypeName']}
- **Probability:** {risk['RiskProbabilityName']}
- **Impact:** {risk['RiskImpactName']}
- **Exposure:** {risk['RiskExposure']}
- **Review Date:** {risk['ReviewDate']}
"""
    return risk_info

def format_test_run(test_run) -> str:
    test_run_info = f"""
## Test Run [TR:{test_run['TestRunId']}] - {test_run['Name']}
- **Status:** {get_execution_status_name(test_run['ExecutionStatusId'])}
- **Test Case:** TC:{test_run['TestCaseId']}
- **Test Set:** TX:{test_run['TestSetId']}
- **Release:** {test_run['ReleaseVersionNumber']}
- **Start Date:** {test_run['StartDate']}
- **End Date:** {test_run['EndDate']}
"""
    return test_run_info

def format_automation_host(host) -> str:
    host_info = f"""
## Automation Host [AH:{host['AutomationHostId']}] - {host['Name']}
{'' if host['Description'] is None else host['Description']}
- **Token:** {host['Token']}
- **Active:** {host['Active']}
- **Last Contact:** {host['LastContactDate']}
"""
    return host_info

def format_capability(capability) -> str:
    """
    Format a program capability into a readable string.
    
    Args:
        capability: The capability data from the API
        
    Returns:
        Formatted string containing the capability information
    """
    capability_info = f"""
## Capability [CP:{capability['CapabilityId']}] - {capability['Name']}
{'' if capability['Description'] is None else capability['Description']}
- **Status:** {capability['StatusName']}
- **Type:** {capability['TypeName']}
- **Priority:** {capability['PriorityName']}
- **% Complete:** {capability['PercentComplete']}%
- **Milestone:** {capability['MilestoneName']}
- **# Requirements:** {capability['RequirementCount']}
"""
    return capability_info



from mcp.server.fastmcp import FastMCP


def register_prompt(mcp: FastMCP) -> None:
    
    @mcp.prompt(name="Create Conventions File", 
                description="Create a starting conventions file Inflectra Spira")
    def create_conventions_file() -> str:
        """
        Create a starting conventions file for Inflectra Spira.
        
        Use this prompt when you need to:
        - Generate a conventions file for Inflectra Spira
        - Get a template for project conventions
        - Start defining project standards and guidelines
        
        Returns:
            A formatted conventions file template
        """
        
        
        return """Create a concise Inflectra Spira conventions file to 
    serve as a quick reference for our environment. 
    This should capture all important patterns and structures 
    while remaining compact enough for an LLM context.

Using the available Inflectra Spira tools, please:

1. Get an overview of ALL programs (get_programs)
2. For ALL programs:
   - Get a list of all the milestones in the program (get_milestones)
   - Get a list of all the products in the program (get_program_products)
   - Get the list of releases in the product (get_releases) 
3. Capture template configuration for EACH product:
   - Template name and ID (get_product_template)
   - Artifact types and sub-types (get_artifact_types)
   - For each artifact type, get list of custom properties
     (get_custom_properties) and clearly identify mandatory fields

Create a concise markdown document with these sections:

1. **Workspaces**: 
    List of all programs and their products
2. **Template Configuration**: 
    Name, ID and description of product template
    List of all artifact types (requirement, test case, etc.),
    with the sub-types for each artifact nested underneath 
3. **Custom Properties**: 
    List of custom properties for each artifact type in the template
4. **Classification Structure**: 
    List of milestones in each program
    List of releases in each product

Focus on identifying and documenting patterns and 
variations between products. 
When listing field names or other details, prioritize the most important ones.
The goal is to create a reference that captures key conventions 
while staying concise."""
    


"""
Provides operations for working with the Spira product releases

This module provides MCP tools for retrieving and updating product releases
"""

from mcp_server_spira.features.formatting import format_release
from mcp_server_spira.features.common import get_spira_client

def _get_releases_impl(spira_client, product_id: int) -> str:
    """
    Implementation of retrieving the list of releases in the specified product

    Args:
        spira_client: The Inflectra Spira API client instance
        product_id: The numeric ID of the product. If the ID is PR:45, just use 45. 
                
    Returns:
        Formatted string containing the list of releases
    """
    try:
        # Get the list of releases in the product
        releases_url = "projects/" + str(product_id) + "/releases?active_only=true"
        releases = spira_client.make_spira_api_get_request(releases_url)

        if not releases:
            return "There are no releases for the product."

        # Format the releases into human readable data
        formatted_results = []
        for release in releases:
            release_info = format_release(release)
            formatted_results.append(release_info)

        return "\n\n".join(formatted_results)
    
    except Exception as e:
        return f"There was a problem using this tool: {e}"

def register_tools(mcp) -> None:
    """
    Register my work tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """

    @mcp.tool()
    def get_releases(product_id: int) -> str:
        """
        Retrieves a list of the releases in the specified product
        
        Use this tool when you need to:
        - View the list of releases in the specified product
        - Get information about multiple releases at once
        - Access the full description and selected fields of releases

        Args:
            product_id: The numeric ID of the product. If the ID is PR:45, just use 45. 
        
        Returns:
            Formatted string containing comprehensive information for the
            requested list of releases, including name, id, description and key fields,
            formatted as markdown with clear section headings
        """
        try:
            spira_client = get_spira_client()
            return _get_releases_impl(spira_client, product_id)
        except Exception as e:
            return f"Error: {str(e)}"
        


"""
Workspace tools for Spira by Inflectra
"""
from mcp_server_spira.features.workspaces.tools import (
    products, programs, product_templates
)


def register_tools(mcp) -> None:
    """
    Register all workspace tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """
    products.register_tools(mcp)
    programs.register_tools(mcp)
    product_templates.register_tools(mcp)
    


"""
Provides operations for working with the Spira product test cases

This module provides MCP tools for retrieving and updating product test cases
"""

from mcp_server_spira.features.formatting import format_test_case
from mcp_server_spira.features.formatting import format_test_case_folder
from mcp_server_spira.features.common import get_spira_client

def _get_test_cases_impl(spira_client, product_id: int) -> str:
    """
    Implementation of retrieving the list of test cases in the specified product, grouped by folder

    Args:
        spira_client: The Inflectra Spira API client instance
        product_id: The numeric ID of the product. If the ID is PR:45, just use 45. 
                
    Returns:
        Formatted string containing the list of test cases, grouped by folder
    """
    try:
        formatted_results = []

        # First we need to get any test cases not in a folder
        _get_test_cases_in_folder(spira_client, product_id, formatted_results, None)

        # Get the entire test case folder hierarchy in the product
        test_case_folders_url = f"projects/{product_id}/test-folders"
        test_case_folders = spira_client.make_spira_api_get_request(test_case_folders_url)

        # Loop through all the test case folders and format
        for test_case_folder in test_case_folders:
            test_case_folder_info = format_test_case_folder(test_case_folder)
            formatted_results.append(test_case_folder_info)

            # Now get all of the test cases in the folder
            _get_test_cases_in_folder(spira_client, product_id, formatted_results, test_case_folder)

        return "\n\n".join(formatted_results)
    
    except Exception as e:
        return f"There was a problem using this tool: {e}"

def _get_test_cases_in_folder(spira_client, product_id: int, formatted_results: list[str], test_case_folder):
    """
    Implementation of retrieving the list of test cases in the specified product, grouped by folder

    Args:
        spira_client: The Inflectra Spira API client instance
        product_id: The numeric ID of the product. If the ID is PR:45, just use 45. 
                
    Returns:
        Formatted string containing the list of test cases, grouped by folder
    """
    try:
        # Get the test case folder id
        test_case_folder_id = 'null'
        release_id = 'null'
        if test_case_folder:
            test_case_folder_id = str(test_case_folder['TestCaseFolderId'])

        # Get the test cases in the folder
        test_cases_url = f"projects/{product_id}/test-folders/{test_case_folder_id}/test-cases?starting_row=1&number_of_rows=1000&sort_field=Name&sort_direction=ASC&release_id={release_id}"
        test_cases = spira_client.make_spira_api_get_request(test_cases_url)

        # Loop through all the test cases and format
        for test_case in test_cases:
            test_case_info = format_test_case(test_case)
            formatted_results.append(test_case_info)

    except Exception as e:
        raise Exception(f"Error returned when getting the test cases in the folder. The error message was: {e}")

def register_tools(mcp) -> None:
    """
    Register product test cases tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """

    @mcp.tool()
    def get_test_cases(product_id: int) -> str:
        """
        Retrieves a list of the test cases in the specified product, grouped by folder
        
        Use this tool when you need to:
        - View the list of test cases in the specified product
        - Get information about multiple test cases at once
        - Access the full description and selected fields of test cases

        Args:
            product_id: The numeric ID of the product. If the ID is PG:45, just use 45. 
        
        Returns:
            Formatted string containing comprehensive information for the
            requested list of test cases, including name, id, description and key fields,
            grouped by the test case folder, formatted as markdown with clear section headings
        """
        try:
            spira_client = get_spira_client()
            return _get_test_cases_impl(spira_client, product_id)
        except Exception as e:
            return f"Error: {str(e)}"



"""
Inflectra Spira MCP Server

A simple MCP server that exposes Inflectra Spira capabilities.

Prerequisites: You need to have the following environment variables defined:

- INFLECTRA_SPIRA_BASE_URL: The base URL to your Spira instance (e.g. https://mycompany.spiraservice.net)
- INFLECTRA_SPIRA_USERNAME: The login to your Spira instance
- INFLECTRA_SPIRA_API_KEY: The API Key (RSS Token) for your Spira instance

"""
import argparse

from mcp.server.fastmcp import FastMCP

from mcp_server_spira.features import register_all
from mcp_server_spira.utils import register_all_prompts

# Create a FastMCP server instance with a name
mcp = FastMCP("inflectra-spira")

# Register all features
register_all(mcp)
register_all_prompts(mcp)

def main():
    """Entry point for the command-line script."""
    parser = argparse.ArgumentParser(
        description="Run the Inflectra Spira MCP server")
    # Add more command-line arguments as needed
    
    parser.parse_args()  # Store args if needed later
    
    # Start the server
    mcp.run()

if __name__ == "__main__":
    main()


"""
Product artifact tools for Spira by Inflectra
"""
from mcp_server_spira.features.productartifacts.tools import (
    releases,
    requirements,
    incidents,
    tasks,
    risks,
    testruns,
    automationhosts,
    testcases,
    testsets
)


def register_tools(mcp) -> None:
    """
    Register all product artifact tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """
    releases.register_tools(mcp)
    requirements.register_tools(mcp)
    incidents.register_tools(mcp)
    tasks.register_tools(mcp)
    risks.register_tools(mcp)
    testruns.register_tools(mcp)
    automationhosts.register_tools(mcp)
    testcases.register_tools(mcp)
    testsets.register_tools(mcp)



"""
Inflectra Spira general utilities.

This module provides helper functions for working with data in Spira
"""

def get_execution_status_name(execution_status_id: int) -> str:
    """
    Gets the name of a Spira test execution status from its ID

    Args:
        execution_status_id: The ID of the execution status
                
    Returns:
        The name of the matching execution status or an empty string, if no match
    """

    if execution_status_id == None:
        return ""
    
    # Find the matching status id
    # 1 = Failed, 2 = Passed, 3 = Not Run, 4 = N/A, 5 = Blocked and 6 = Caution
    match execution_status_id:
        case 1:
            return "Failed"
        case 2:
            return "Passed"
        case 3:
            return "Not Run"
        case 4:
            return "N/A"
        case 5:
            return "Blocked"
        case 6:
            return "Caution"

    return "(Unknown)"


"""
Common utilities for Inflectra Spira features.

This module provides shared functionality used by both tools and resources.
"""
from mcp_server_spira.utils.spira_client import SpiraClient, get_client

def get_spira_client() -> SpiraClient:
    """
    Get the Spira API client.
    
    Returns:
        SpiraClient instance
        
    """
    # Get Spira client
    spira_client = get_client()        
    return spira_client



"""
Provides operations for working with the Spira product test runs

This module provides MCP tools for retrieving and updating product test runs
"""

from mcp_server_spira.features.formatting import format_test_run
from mcp_server_spira.features.common import get_spira_client

def _get_test_runs_impl(spira_client, product_id: int) -> str:
    """
    Implementation of retrieving the list of test runs in the specified product

    Args:
        spira_client: The Inflectra Spira API client instance
        product_id: The numeric ID of the product. If the ID is PR:45, just use 45. 
                
    Returns:
        Formatted string containing the list of test runs
    """
    try:
        # Get the list of test runs in the product
        starting_row = 1
        number_of_rows = 500  # Only show first 500 test runs
        sort_field = "EndDate"
        sort_direction = "DESC"
        test_runs_url = f"projects/{product_id}/test-runs?starting_row={starting_row}&number_of_rows={number_of_rows}&sort_field={sort_field}&sort_direction={sort_direction}"
        test_runs = spira_client.make_spira_api_get_request(test_runs_url)

        if not test_runs:
            return "There are no test runs for the product."

        # Format the test runs into human readable data
        formatted_results = []
        for test_run in test_runs:
            test_run_info = format_test_run(test_run)
            formatted_results.append(test_run_info)

        return "\n\n".join(formatted_results)
    
    except Exception as e:
        return f"There was a problem using this tool: {e}"

def register_tools(mcp) -> None:
    """
    Register product test runs tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """

    @mcp.tool()
    def get_test_runs(product_id: int) -> str:
        """
        Retrieves a list of the test runs in the specified product
        
        Use this tool when you need to:
        - View the list of test runs in the specified product
        - Get information about multiple test runs at once
        - Access the full description and selected fields of test runs

        Args:
            product_id: The numeric ID of the product. If the ID is PR:45, just use 45. 
        
        Returns:
            Formatted string containing comprehensive information for the
            requested list of test runs, including name, id, description and key fields,
            formatted as markdown with clear section headings
        """
        try:
            spira_client = get_spira_client()
            return _get_test_runs_impl(spira_client, product_id)
        except Exception as e:
            return f"Error: {str(e)}"



"""
Provides operations for working with the Spira program capabilities

This module provides MCP tools for retrieving program capabilities
"""

from mcp_server_spira.features.formatting import format_capability
from mcp_server_spira.features.common import get_spira_client

def _get_capabilities_impl(spira_client, program_id: int) -> str:
    """
    Implementation of retrieving the list of capabilities in the specified program

    Args:
        spira_client: The Inflectra Spira API client instance
        program_id: The numeric ID of the program. If the ID is PG:45, just use 45. 
                
    Returns:
        Formatted string containing the list of capabilities
    """
    try:
        # Get the list of capabilities in the program
        capabilities_url = f"programs/{program_id}/capabilities/search?current_page=1&page_size=500"
        capabilities = spira_client.make_spira_api_get_request(capabilities_url)

        if not capabilities:
            return "There are no capabilities in the specified program."

        # Format the capabilities into human readable data
        formatted_results = []
        for capability in capabilities:
            capability_info = format_capability(capability)
            formatted_results.append(capability_info)

        return "\n\n".join(formatted_results)
    
    except Exception as e:
        return f"There was a problem using this tool: {e}"

def register_tools(mcp) -> None:
    """
    Register program capabilities tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """

    @mcp.tool()
    def get_capabilities(program_id: int) -> str:
        """
        Retrieves a list of the capabilities in the specified program
        
        Use this tool when you need to:
        - View the list of capabilities in the specified program
        - Get information about multiple capabilities at once
        - Access the full description and selected fields of capabilities

        Args:
            program_id: The numeric ID of the program. If the ID is PG:45, just use 45. 
        
        Returns:
            Formatted string containing comprehensive information for the
            requested list of capabilities, including name, id, description and key fields,
            formatted as markdown with clear section headings
        """
        try:
            spira_client = get_spira_client()
            return _get_capabilities_impl(spira_client, program_id)
        except Exception as e:
            return f"Error: {str(e)}"



"""
MCP Inflectra Spira Server - A Model Context Protocol server for Spira (SpiraTest, SpiraTeam and SpiraPlan)
integration.
"""

import importlib.metadata

try:
    __version__ = importlib.metadata.version("mcp-server-spira")
except importlib.metadata.PackageNotFoundError:
    __version__ = "0.0.0"


"""
Provides operations for working with the Spira program milestones

This module provides MCP tools for retrieving and updating program milestones
"""

from mcp_server_spira.features.formatting import format_milestone
from mcp_server_spira.features.common import get_spira_client

def _get_milestones_impl(spira_client, program_id: int) -> str:
    """
    Implementation of retrieving the list of milestones in the specified program

    Args:
        spira_client: The Inflectra Spira API client instance
        program_id: The numeric ID of the program. If the ID is PG:45, just use 45. 
                
    Returns:
        Formatted string containing the list of milestones
    """
    try:
        # Get the list of milestones in the program
        milestones_url = "programs/" + str(program_id) + "/milestones"
        milestones = spira_client.make_spira_api_get_request(milestones_url)

        if not milestones:
            return "There are no milestones in the current program."

        # Format the milestones into human readable data
        formatted_results = []
        for milestone in milestones:
            milestone_info = format_milestone(milestone)
            formatted_results.append(milestone_info)

        return "\n\n".join(formatted_results)
    
    except Exception as e:
        return f"There was a problem using this tool: {e}"

def register_tools(mcp) -> None:
    """
    Register my work tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """

    @mcp.tool()
    def get_milestones(program_id: int) -> str:
        """
        Retrieves a list of the milestones in the specified program
        
        Use this tool when you need to:
        - View the list of milestones in the specified program
        - Get information about multiple milestones at once
        - Access the full description and selected fields of milestones

        Args:
            program_id: The numeric ID of the program. If the ID is PG:45, just use 45. 
        
        Returns:
            Formatted string containing comprehensive information for the
            requested list of milestones, including name, id, description and key fields,
            formatted as markdown with clear section headings
        """
        try:
            spira_client = get_spira_client()
            return _get_milestones_impl(spira_client, program_id)
        except Exception as e:
            return f"Error: {str(e)}"
        


from mcp_server_spira.utils.conventions_prompt import register_prompt


def register_all_prompts(mcp):
    """
    Register prompts with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """
    # Register prompts here
    register_prompt(mcp)
    


# Inflectra Spira MCP features package
from mcp_server_spira.features import (
    mywork, productartifacts, programartifacts, templateconfiguration, workspaces, automation
)



def register_all(mcp):
    """
    Register all features with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """
    mywork.register(mcp)
    productartifacts.register(mcp)
    programartifacts.register(mcp)
    templateconfiguration.register(mcp)
    workspaces.register(mcp)
    automation.register(mcp)



"""
Main entry point for running the MCP Inflectra Spira server as a module.

This allows running the server with:
    python -m mcp_server_spira
"""
from mcp_server_spira.server import main

if __name__ == "__main__":
    main()


"""
Automation tools for Spira by Inflectra
"""
from mcp_server_spira.features.automation.tools import (
    automatedtestruns, builds
)


def register_tools(mcp) -> None:
    """
    Register all automation tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """
    automatedtestruns.register_tools(mcp)
    builds.register_tools(mcp)
    


"""
Defines the entry point of the extension
"""

import setuptools
import os
import codecs
import io

with io.open("README.md") as readme_file:
    long_description = readme_file.read()

# Register plugin
setuptools.setup(
    name ='mcp-server-spira',
    version = '1.0.0',
    author = 'Inflectra Corporation',
    author_email ='support@inflectra.com',
    url = 'https://github.com/Inflectra/mcp-server-spira',
    description = 'A Model Context Protocol (MCP) server enabling AI assistants to interact with Spira by Inflectra.',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    packages = setuptools.find_packages(where='src'),
    package_dir={'': 'src'},
    classifiers = [
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    entry_points = {
        'console_scripts': [
            'mcp-server-spira = mcp_server_spira.server:main',
        ],
    },
    include_package_data=True,
    package_data={
        '': ['*.md', '*.txt', '*.cfg', '*.json', '*.yaml', '*.yml'],
        'mcp_server_spira': ['**/*.py', '**/*.md', '**/*.json', '**/*.yaml', '**/*.yml', '**/*.cfg'],
    },
)



"""
Provides operations for recording automated test run results into Spira

This module provides MCP tools for recording the results of automated
test against a matching test case in Spira 
"""

import datetime

from mcp_server_spira.features.common import get_spira_client

def _record_automated_test_run_impl(spira_client, product_id: int, test_name: str, short_message: str, long_message: str, error_count: int, test_case_id: int, execution_status_id: int) -> str:
    """
    Records an automated test result in Spira

    Args:
        spira_client: The Inflectra Spira API client instance
        product_id: The numeric ID of the product. If the ID is PG:45, just use 45.
        test_name: The name of the test being run
        short_message: A short description (50 characters or less) of the result of the test execution
        long_message: The full description of the testing outcome, in plain text format
        error_count: The number of errors that happened during the test (0 if none)
        test_case_id: The ID of the test case in Spira being executed, without the TC prefix (e.g. TC:12 would be 12)
        execution_status_id: The ID of the execution status of the test (1 = Failed, 2 = Passed, 3 = Not Run, 4 = N/A, 5 = Blocked and 6 = Caution)
                
    Returns:
        The ID of the new test run that was created (with 'TR' prefix)
    """
    try:
        # Make the start/end time right now
        start_time = datetime.datetime.now()
        end_time = datetime.datetime.now()

        # The body we are sending
        body = {
            # Constant for plain text
            'TestRunFormatId': 1,
            'StartDate': start_time.strftime('%Y-%m-%dT%H:%M:%SZ'),
            'EndDate': end_time.strftime('%Y-%m-%dT%H:%M:%SZ'),
            'RunnerName': "MCP Server",
            'RunnerTestName': test_name,
            'RunnerMessage': short_message,
            'RunnerStackTrace': long_message,
            'RunnerAssertCount': error_count,
            'TestCaseId': test_case_id,
            # Passes (2) if the stack trace length is 0
            'ExecutionStatusId': execution_status_id
        }

        # Record the test run using the API method
        record_automated_url = "projects/" + str(product_id) + "/test-runs/record "
        testrun = spira_client.make_spira_api_post_request(record_automated_url, body)

        if not testrun:
            return "The test run was not recorded successfully."

        # Extract the new test run
        test_run_id = testrun['TestRunId']

        return "TR:" + str(test_run_id)
    except Exception as e:
        return f"There was a problem using this tool: {e}"
    
def register_tools(mcp) -> None:
    """
    Register tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """

    @mcp.tool()
    def record_automated_test_run(product_id: int, test_name: str, short_message: str, long_message: str, error_count: int, test_case_id: int, execution_status_id: int) -> str:
        """
        Records an automated test result in Spira
        
        Use this tool when you need to:
        - Push the results of an automated software test into Spira
                    
        Args:
            product_id: The numeric ID of the product. If the ID is PG:45, just use 45.
            test_name: The name of the test being run
            short_message: A short description (50 characters or less) of the result of the test execution
            long_message: The full description of the testing outcome, in plain text format
            error_count: The number of errors that happened during the test (0 if none)
            test_case_id: The ID of the test case in Spira being executed, without the TC prefix (e.g. TC:12 would be 12)
            execution_status_id: The ID of the execution status of the test (1 = Failed, 2 = Passed, 3 = Not Run, 4 = N/A, 5 = Blocked and 6 = Caution)

        Returns:
            The ID of the newly created test run in Spira, with a TR prefix added (for example TR:123)
        """
        try:
            spira_client = get_spira_client()
            return _record_automated_test_run_impl(spira_client, product_id, test_name, short_message, long_message, error_count, test_case_id, execution_status_id)
        except Exception as e:
            return f"Error: {str(e)}"
        


"""
Provides operations for working with the Spira product template workspace

This module provides MCP tools for retrieving and updating product templates (also known as projects).
"""

from mcp_server_spira.features.formatting import format_product_template
from mcp_server_spira.features.common import get_spira_client

def _get_product_templates_impl(spira_client) -> str:
    """
    Implementation of retrieving the list of Spira product templates
    the current user has access to

    Args:
        spira_client: The Inflectra Spira API client instance
                
    Returns:
        Formatted string containing the list of available product templates
    """
    try:
        # Get the list of available product templates for the current user
        product_templates_url = "project-templates"
        product_templates = spira_client.make_spira_api_get_request(product_templates_url)

        if not product_templates:
            return "The are no product templates visible to the current user."

        # Format the product templates into human readable data
        formatted_results = []
        for product_template in product_templates[:100]:  # Only show first 100 product templates
            product_template_info = format_product_template(product_template)
            formatted_results.append(product_template_info)

        return "\n\n".join(formatted_results)
    except Exception as e:
        return f"There was a problem using this tool: {e}"

def _get_product_template_impl(spira_client, template_id: int) -> str:
    """
    Implementation of retrieving a single Spira product template

    Args:
        spira_client: The Inflectra Spira API client instance
        template_id: The numeric ID of the product template. If the ID is PT:45, just use 45.
                
    Returns:
        Formatted string containing the details of the requested product template
    """
    try:
        # Get the product template by its ID
        product_templates_url = "project-templates/" + str(template_id)
        product_template = spira_client.make_spira_api_get_request(product_templates_url)

        if not product_template:
            return "Unable to fetch product template details for ID " + str(template_id) + "."

        # Format the product template into human readable data
        product_template_info = format_product_template(product_template)
        return product_template_info
    except Exception as e:
        return f"There was a problem using this tool: {e}"
    
def register_tools(mcp) -> None:
    """
    Register my work tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """

    @mcp.tool()
    def get_product_templates() -> str:
        """
        Retrieves a list of the product templates that the current user has access to
        
        Use this tool when you need to:
        - View the list of product templates that a user has access to
        - Get information about multiple product templates at once
        - Access the full description and selected fields of product templates
                    
        Returns:
            Formatted string containing comprehensive information for the
            requested list of product templates, including name, id, description and key fields,
            formatted as markdown with clear section headings
        """
        try:
            spira_client = get_spira_client()
            return _get_product_templates_impl(spira_client)
        except Exception as e:
            return f"Error: {str(e)}"
        
    @mcp.tool()
    def get_product_template(template_id: int) -> str:
        """
        Retrieves a product template by its unique numeric ID (remove any PT prefixes)
        
        Use this tool when you need to:
        - View the details of a product template when you know its ProjectTemplateId
        - Get information about a single product template
        - Access the full description and selected fields of the product template

        Args:
            template_id: The numeric ID of the product template. If the ID is PT:45, just use 45.
                    
        Returns:
            Formatted string containing comprehensive information for the
            requested product template, including name, id, description and key fields,
            formatted as markdown with clear section headings
        """
        try:
            spira_client = get_spira_client()
            return _get_product_template_impl(spira_client, template_id)
        except Exception as e:
            return f"Error: {str(e)}"
        


"""
My assigned work tools for Spira by Inflectra
"""
from mcp_server_spira.features.mywork.tools import (
    mytasks,myincidents,myrequirements,mytestcases,mytestsets
)


def register_tools(mcp) -> None:
    """
    Register all work item tools with the MCP server.
    
    Args:
        mcp: The FastMCP server instance
    """
    mytasks.register_tools(mcp)
    myincidents.register_tools(mcp)
    myrequirements.register_tools(mcp)
    mytestcases.register_tools(mcp)
    mytestsets.register_tools(mcp)
    